#!/bin/bash

api(){
    if [ -z "GITLAB_TOKEN" ]
    then
        . ~/.gitlab-apertis-org-token
    fi

    [ -z "$CI_SERVER_HOST" ] && ( echo "No server host specified, use CI_SERVER_HOST env var to specify tit" >&2 ; exit 1 )

    api_base_url=https://gitlab.apertis.org/api/v4
    api_endpoint=$1

    if echo "$api_endpoint" | grep -q '?'
    then
      items="&per_page=100"
    else
      items="?per_page=100"
    fi
    shift

    curl -s -H "PRIVATE-TOKEN: $GITLAB_TOKEN" --request ${request:-GET} $api_base_url$api_endpoint$items "$@"
}

get-id(){
    echo $1 | sed -e s,/,%2F,g -e s,+,%2B,g
}

create-merge-request(){
    repoid=$(get-id $1)
    
    request=POST api /projects/$repoid/merge_requests -d source_branch=$2 -d target_branch=$3 -d title="Merge changes from $2 into $3"
}

create-repo-branch(){
    repoid=$(get-id $1)

    request=POST api /projects/$repoid/repository/branches -d branch=$2 -d ref=$3
}

create-repo-ci-schedule(){
    repoid=$(get-id $1)
    branch_ref=$(get-id $2)

    request=POST api /projects/$repoid/pipeline_schedules -d description="create CI Schedule" -d ref="$branch_ref" -d cron="0 3 * * *"
}

create-repo-trigger(){
    repoid=$(get-id $1)

    request=POST api /projects/$repoid/triggers -d description="create trigger"
}

delete-repo-branch(){
    repoid=$(get-id $1)
    branchid=$(get-id $2)

    request=DELETE api /projects/$repoid/repository/branches/$branchid
}

get-groups-repos(){
    groupid=$(get-id $1)

    api "/groups/$groupid/projects?with_shared=false&archived=false" | jq -r .[].path
}

get-repo-branch(){
    repoid=$(get-id $1)
    branchid=$(get-id $2)

    api /projects/$repoid/repository/branches/$branchid | jq -r '.commit.short_id'
}

get-repo-branches(){
    repoid=$(get-id $1)

    api /projects/$repoid/repository/branches | jq -r '.[] | [.name, .commit.short_id] | @tsv'
}

get-repo-head(){
    repoid=$(get-id $1)

    api /projects/$repoid | jq -r '.default_branch'
}

get-repo-path(){
    repoid=$(get-id echo pkg/$1)
    path=$(api /projects/$repoid | jq -r '.path_with_namespace')

    if [ "$path" != null ]; then
        echo $path
    fi
}

get-repo-tags(){
    repoid=$(get-id $1)

    api /projects/$repoid/repository/tags | jq -r '.[] | [.name, .commit.short_id] | @tsv'
}

get-repo-triggers(){
    repoid=$(get-id $1)

    api /projects/$repoid/triggers | jq -r '.[] | [.token] | @tsv'
}

merge-mr(){
    repoid=$(get-id $1)
    mr_id=$2

    request=PUT api /projects/$repoid/merge_requests/$mr_id/merge -d merge_commit_message="Auto merge changes for this MR, if the pipeline succeeds" -d merge_when_pipeline_succeeds="true" -d should_remove_source_branch="true"
}

trigger-pipeline(){
    # Trigger pipeline on selected package

    set -e

    usage () {
        echo "Usage: $0 <package> <pipeline>"
        echo ""
        echo "Example: $0 libfoo debian/sid"
    }

    if [ "$#" -ne 2 ]; then
        usage
        exit 1
    fi

    pkg_path=$(get-repo-path $1)
    repoid=$(get-id $pkg_path)
    ref=$2

    TRIGGER_TOKEN="$($get_repo_triggers $repoid)"

    if [ -z "$TRIGGER_TOKEN" ]; then
        echo "I don't have a trigger token"
        $create_repo_trigger $repoid
        TRIGGER_TOKEN=$(get-repo-triggers $repoid)
    fi

    request=POST api /projects/$repoid/trigger/pipeline -d ref=$ref -d token=$TRIGGER_TOKEN
}
